from pathlib import Path
from dataclasses import dataclass, asdict
from zona.config import ZonaConfig, find_config
import yaml


@dataclass
class Layout:
    root: Path
    content: Path
    templates: Path
    output: Path

    @classmethod
    def from_input(
        cls, root: Path, output: Path | None = None, validate: bool = True
    ) -> "Layout":
        layout = cls(
            root=root.resolve(),
            content=(root / "content").resolve(),
            templates=(root / "templates").resolve(),
            output=(root / "public").resolve() if not output else output,
        )
        if validate:
            for path in [layout.content, layout.templates]:
                if not path.is_dir():
                    raise FileNotFoundError(
                        f"Missing required directory: {path}"
                    )
        return layout


def discover_layout(
    cli_root: Path | None = None, cli_output: Path | None = None
) -> Layout:
    if cli_root:
        root = cli_root
    else:
        config = find_config(cli_root)
        if config:
            root = config.parent
        else:
            root = Path.cwd()
    return Layout.from_input(root, cli_output)


def initialize_site(root: Path | None = None):
    # initialize a new project
    if not root:
        root = Path.cwd()
    root = root.absolute().resolve()
    config = find_config(root)
    if config is not None:
        raise FileExistsError(f"Config file already exists at {config}")
    # create requires layout
    layout = Layout.from_input(root=root, validate=False)
    for dir in [layout.root, layout.content, layout.templates]:
        if not dir.is_dir():
            dir.mkdir()
    config_path = layout.root / "config.yml"
    config = ZonaConfig()
    with open(config_path, "w") as f:
        yaml.dump(
            asdict(config),
            f,
            sort_keys=False,
            default_flow_style=False,
            indent=2,
        )
