from datetime import date
import fnmatch
from pathlib import Path
from shutil import copy2
import string

from dacite import Config, DaciteError, from_dict
import frontmatter
from yaml import YAMLError

from zona.models import Metadata


def ensure_parents(target: Path):
    """Ensure the target's parent directories exist."""
    target.parent.mkdir(parents=True, exist_ok=True)


def copy_static_file(src: Path, dst: Path):
    """Copy a static file from one location to another."""
    ensure_parents(dst)
    copy2(src, dst)


def filename_to_title(path: Path) -> str:
    name = path.stem
    words = name.replace("-", " ").replace("_", " ")
    return string.capwords(words)


def normalize_url(url: str) -> str:
    if not url.startswith("/"):
        url = "/" + url
    return url


def should_ignore(path: Path, patterns: list[str], base: Path) -> bool:
    rel_path = path.relative_to(base)
    return any(fnmatch.fnmatch(str(rel_path), pattern) for pattern in patterns)


def parse_metadata(path: Path) -> tuple[Metadata, str]:
    """
    Parses a file and returns parsed Metadata and its content. Defaults
    are applied for missing fields. If there is no metadata, a Metadata
    with default values is returned.

    Raises:
        ValueError: If the metadata block is malformed in any way.
    """
    try:
        post = frontmatter.load(str(path))
    except YAMLError as e:
        raise ValueError(f"YAML frontmatter error in {path}: {e}")
    raw_meta = post.metadata or {}
    defaults = {
        "title": filename_to_title(path),
        "date": date.fromtimestamp(path.stat().st_mtime),
    }
    meta = {**defaults, **raw_meta}
    try:
        metadata = from_dict(
            data_class=Metadata,
            data=meta,
            config=Config(check_types=True, strict=True),
        )
    except DaciteError as e:
        raise ValueError(f"Malformed metadata in {path}: {e}")
    return metadata, post.content
